#!/usr/bin/env bash

# This is a medium advanced example bash script to show how to use GForm to create a GUI
# and then have your shell script act upon the input by sending commands back to the GUI
# to modify the data.

# It shows a label and a text box in readonly mode on the top row 
# the next row has a ListBox to show the help file names.
# the next row has a TextView to show the files selected by the above ListBox
# 
if [ -e "/tmp/fifo2" ]; then rm /tmp/fifo2; fi
if [ -e "/tmp/fifo1" ]; then rm /tmp/fifo1; fi
CurDir=$(pwd)
CurDir=${CurDir%/*}
# Set up our list using dir to get contents of help dir and run Gform.gambas with some args.
IFS=$'\n'
HDIR="$CurDir/help"
if [ ! -e "$HDIR/2 Runtime args.txt" ]; then
 HDIR="$CurDir"
 if [ ! -e "$HDIR/2 Runtime args.txt" ]; then
  gbr3 ../GForm quiet toponly title="Notice.." label="|Help Files were not found|\nPlease download them from the GForm github\nto use this example script.\nOr use GForm help or GForm fullhelp\nAs these texts are now built in." button="|Okay|close icon=ok" 2>/dev/null
  exit
 fi
fi
FILES=$(dir -1N "$HDIR/")
read -d '{' -a NameArray <<< "$FILES{"
for s in "${NameArray[@]}"; do txt="$txt,$s"; done
NameList=${txt#,*}

# The following is one long line split using \ to make things clearer
# as each line is each row in the form. note, using option 'quiet' to
# suppress any stdout message unlike the 'Test_GForm (simple).sh' example.

../GForm quiet font="Carlito,14,Italic" width=scr-200 pipe="/tmp/fifo1" listen="/tmp/fifo2" title="GForm medium advanced shell script example" \
box label="|Current File" input="TextBox1||readonly left" unbox \
listbox="ListBox1|$NameList||nostretch lines=6 background=200,220,220" \
textarea="TextArea1|Help will be here\nJust select a topic..|stretch readonly lines=10 background=210,220,210" \
box label="|Change reader font" fontbox="||stretch linkto=TextArea1" \
fbutton="lft||icon=text-left" fbutton="ctr||icon=text-center" fbutton="rgt||icon=text-right" spring button="BQ|Quit|close icon=quit" 2>/dev/null&false

 while [ ! -e "/tmp/fifo1" ]; do
 sleep 0.1
 done
AppOpen=1


CleanUp() {
exec 3<&- 2>/dev/null # close the pipe handle
if [ -e "/tmp/fifo2" ]; then
rm /tmp/fifo2
fi
if [ -e "/tmp/fifo1" ]; then
rm /tmp/fifo1
fi
exit
}

Alert() {  
# just using GForm to pop up a message box. 
# it uses the running GUI's internal "message" function
if [ $AppOpen -eq 0 ]; then
gbr3 GForm quiet toponly title="Notice.." label="|$1" button="|Okay|close icon=ok" 2>/dev/null
return
fi
SendE "message=$1"
}

Send() { echo -e "$1" >>/tmp/fifo2; } # use Send to send multiple lines with '\n'
SendE() { echo -E "$1" >>/tmp/fifo2; } # use SendE to send one line containing '\n'


TextToData() {
PT=$1
CName=${PT%%|*}   # get the name of the control talking to us.
TMP=${PT#*|}
if [[ "$TMP" = *"|"* ]]; then
 CText=${TMP%|*}        # see if rest of message has another field.
 CData=${TMP#*|}        # Toggle buttons, checkboxes, etc have other info
else
 CText="$TMP"; CData=""
fi
}


DoCommand() {  
# this is the main function for processing the data coming from the GUI.
# The message will be in the form of "object_name|object_text|object_value"

if [ "$CName" = "ListBox1" ]; then  # the listbox item changed so send the "filetext" command to the GUI
 ListIndex=$CData
 TXT=$(cat "$HDIR/$CText")
 Send "settext=TextBox1|$CText\nfiletext=TextArea1|$HDIR/$CText"

# The Font changer was given the flag "linkto=TextArea1" so no code needed it's auto attached.

# next is our text alignment buttons..
elif [ "$CName" = "lft" ]; then  Send "align=TextArea1|l" 
elif [ "$CName" = "ctr" ]; then  Send "align=TextArea1|c" 
elif [ "$CName" = "rgt" ]; then  Send "align=TextArea1|r" 

elif [ "$CName" = "BQ" ]; then  CleanUp 

else
Alert "unknown message\n$1" "w"
fi

}


# Main()
# This is the main loop, It opens the pipe and if a line of text comes
# through it reads it to $Pipetext and runs the DoCommand() procedure above.
# Closing the GUI deletes the pipe file so our loop runs while the file exists.

exec 3</tmp/fifo1 
  while [ -e "/tmp/fifo1" ]; do
  read -u 3 PipeText
   if [ ! -z "$PipeText" ]; then  # We got some text so process it.
   TextToData $PipeText
    DoCommand $PipeText
   fi
  done

CleanUp  # clean up and exit.


